# -*- coding: utf-8 -*-
# (C) Copyright 1996-2017 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

#! /usr/bin/env python

from __future__ import print_function
from argparse import ArgumentParser
import re
import time

INDENT = "    "

supported = [
    'void',
    'char',
    'short',
    'int',
    'long',
    'size_t',
    'float',
    'double',
    '_Bool',
]

c2ftype = {
    'char':'character(c_char)',
    'short':'integer(c_short)',
    'int':'integer(c_int)',
    'long':'integer(c_long)',
    'size_t':'integer(c_size_t)',
    'float':'real(c_float)',
    'double':'real(c_double)',
    '_Bool':'logical(c_bool)',
}

ftype2use = {
    'character(c_char)':'c_char',
    'integer(c_short)':'c_short',
    'integer(c_int)':'c_int',
    'integer(c_long)':'c_long',
    'integer(c_size_t)':'c_size_t',
    'real(c_float)':'c_float',
    'real(c_double)':'c_double',
    'logical(c_bool)':'c_bool'
}

function_signature = re.compile('''^
(                                            #1 Type
  (const\s+)?                                #2 Leading const
  ([^\*\&\s\[]+)                             #3
  \s*
  (\&|\*)?                                   #4
  \s*
  (const)?                                   #5
  \s*
  (\&|\*)?                                   #6
  \s*
)
(.+?)\s*                                   #7 Function name
\((.*?)\)$                                 #8 Argument list
''',re.VERBOSE)

class ParsingFailed(Exception):
    def __init__(self,msg):
        Exception.__init__(self, msg)

class Argument:

    arg_signature = re.compile('''^
    (                                            #1 Type
      (const\s+)?                                #2 Leading const
      ([^\*\&\s\[]+)                             #3
      \s*
      (\&|\*)?                                   #4
      \s*
      (const)?                                   #5
      \s*
      (\&|\*)?                                   #6
      \s+?
      (\&|\*)?                                   #7
    )
    (.+?)      #8
    (\[\])?$   #9
    ''',re.VERBOSE)

    def __init__(self,arg,debug=False):
        self.c = arg
        self.type=""
        self.value=""
        self.ref=False
        
        match = Argument.arg_signature.match(arg)
        if not match:
            raise ParsingFailed("Could not parse argument "+arg)

        self.type  = str(match.group(1)).strip()
        self.base_type  = str(match.group(3)).strip()
        self.name = str(match.group(8)).strip()
        if( match.group(9) and self.base_type in supported ):
            self.array = True
        else:
            self.array = False
        
        if( "&" in self.type ):
            self.ref = True
        else:
            self.ref = False
        
        if( "*" in self.type ):
            self.ptr = True
        else:
            self.ptr = False
        
        if debug:
            print ("name = ",self.name)
            print ("type = ",self.type)
            print ("base_type = ",self.base_type)
            print ("array = ",self.array)
            print ("ref = ",self.ref)
            print ("ptr = ",self.ptr)
 
        self.use = set()
        self.fortran = ""
        if( self.base_type == "char" and self.ptr and not self.ref  ) :
            self.fortran += "character(c_char), dimension(*)"
            self.use.add("c_char")
        else:
            if(self.ptr):
                self.fortran += "type(c_ptr)"
                self.use.add("c_ptr")
            else:
                try:
                    ftype = c2ftype[self.base_type]
                    self.fortran += ftype
                    self.use.add(ftype2use[ftype])
                except KeyError:
                    raise ParsingFailed("Could not parse argument "+arg)

            if( self.array ):
                self.fortran += ", dimension(*)"

            elif( not self.ref ):
                self.fortran += ", value"
                
        self.fortran += " :: "+self.name
        
        if( debug ):
            print( self.fortran )

class ReturnType:

    def __init__(self,line):

        match = function_signature.match(line)
        if( not match ):
            raise ParsingFailed("Could not parse return type for line "+line)
        self.type = str(match.group(1)).strip()
        self.base_type = str(match.group(3)).strip()
        self.ptr = match.group(4) or match.group(6)
        self.use = set()
        self.ftype = ""

        if( self.type != "void" ):
            
            if( self.ptr ):
                self.ftype = "type(c_ptr)"
                self.use.add("c_ptr")
            else:
                if( not self.type in supported ):
                    raise ParsingFailed("Cannot return type "+self.type+" for statement "+str(line))
                try:
                    self.ftype = c2ftype[self.base_type]
                    self.use.add(ftype2use[self.ftype])
                except KeyError:
                    raise ParsingFailed("Could not parse return type for statement "+line)

    def __nonzero__(self): # python 2
        return self.type != "void"

    def __bool__(self): # python 3
        return self.type != "void"
        
class Function:
    
    def __init__(self,line):
        self.line = line
        self.return_type = ReturnType(line)
        self.name = self.extract_function_name(line)
        self.arguments = [ Argument(function_arg) for function_arg in self.extract_function_args(line) ]

    def extract_function_name(self,line):
        match = function_signature.match(line)
        if match:
            fn = str(match.group(7))
            return fn

    def extract_function_args(self,line):
        match = function_signature.match(line)
        if match:
            args = str(match.group(8)).strip()
            if args:
                return [arg.strip() for arg in args.split(',')]
        return []

    def fix_fortran_linewidth(self,code,linewidth):
        newcode = ""
        for line in code.split('\n'):
            comment = line.strip()[0] == "!"
            if comment :
                width = linewidth
                end = ''
                begin = '!   '
            else :
                width = linewidth-1
                begin = "  "+"&"
                end = "&"
            while len(line) > width:
                newcode += line[:width] + end + "\n"
                line = begin + line[width:]
            newcode += line + "\n"
        return newcode

    def fortran_interface(self,linewidth=80):
        sep = '!'+'-'*(linewidth-1)
        intf = sep+"\n"
        intf += "! "+self.line+"\n"
        intf += sep+"\n"
        if( self.return_type ):
            intf += "function "
        else:
            intf += "subroutine "
        intf += self.name+"("

        if self.arguments:
            intf += " "+", ".join([arg.name for arg in self.arguments])+" "
        intf += ")"+" bind(C,name=\""+self.name+"\")\n"
        
        self.use = set()
        self.use.update(self.return_type.use)
        for arg in self.arguments:
            self.use.update(arg.use)
        if( self.use ):
            intf += INDENT+"use iso_c_binding, only: "+", ".join(self.use)+"\n"

        
        if( self.return_type ):
            intf += INDENT+self.return_type.ftype + " :: " + self.name + "\n"
        for arg in self.arguments:
            intf += INDENT+arg.fortran + "\n"
        
        if( self.return_type ):
            intf += "end function\n"
        else:
            intf += "end subroutine\n"

        intf += sep
        
        intf = self.fix_fortran_linewidth(intf,linewidth)
        return intf

class Code:
    
    def __init__(self):
        self.content=''
    
    def readline(self,line):
        self.content+=line
    
    def statements(self):
        self.statements = [ statement.strip() for statement in self.content.split(';')[:-1] ]
        return self.statements

# -----------------------------------------------------------------------------

parser = ArgumentParser()
parser.add_argument("file", type=str, help="file to parse")
parser.add_argument("-o", "--output", type=str, help="output")
parser.add_argument("-m", "--module", type=str, help="module")
parser.add_argument("-t", "--types", type=str, help='e.g. {"idx_t":"integer(c_int)","gidx_t":"integer(c_long)"}' )
parsed = parser.parse_args()
input  = parsed.file
output = parsed.output
module = parsed.module

if( parsed.types ) :
    import json
    extra_types = json.loads( str(parsed.types) )
    for k in extra_types:
        supported.append(k)
        c2ftype[k] = extra_types[k]

if not module:
    module = output
    module = str(re.sub(r'(.+)(\..+)',r'\1',module))
    module.replace('-','_')

code = Code()

with open(input,'r') as file:
    content = file.read()
    externC = re.compile('^extern\s+"C"(\s*{)?')
    
    regex_externC = [re.compile('^extern\s+"C"(\s*{)?'),re.compile('^{')]
    in_externC = [False,False]

    for line in content.splitlines():
        line = line.strip()
        
        # Go in extern "C" region
        if not in_externC[0] :
            match = regex_externC[0].match(line)
            if match:
                in_externC[0]=True
                if match.group(1):
                    in_externC[1] = True
                    continue
        if in_externC[0] and not in_externC[1] :
            match = regex_externC[1].match(line)
            if match:
                in_externC[1] = True
                continue

        # Go out of extern "C" region
        if re.search("^}",line):
            in_externC[0] = False
            in_externC[1] = False

        # Skip empty lines and comment lines
        if re.search("^//|^$",line):
            continue
        
        # Line to parse
        if in_externC[1]:
            code.readline(line)

intf = ""
intf += "! Do not modify!\n"
intf += "! Automatically generated file that contains Fortran bindings\n"
intf += "! of C funtions in file "+input+" contained within extern \"C\" scope.\n"
intf += "! Time of creation: "+str(time.asctime(time.localtime()))+"\n\n"
intf += "module "+module+"\n"
intf += "implicit none\n"
intf += "interface\n\n"
for statement in code.statements():
    try:
        intf += Function(statement).fortran_interface()+"\n"
    except ParsingFailed as e:
        print("\n\n"+"-"*80+"\n"+"Automatic generation of Fortran bindings failed for file\n\n"+
              "    "+input+"\n\n"+"ParsingFailed for statement:\n\n"+
              "    "+statement+"\n\nError: "+str(e)+"\n"+"-"*80+"\n")
        raise
intf += "end interface\n"
intf += "end module\n"

with open(output,'w') as out:
    out.write(intf)



# -*- coding: utf-8 -*-
# Program to read NEMO ocean model output temperature field file (netCDF3),
# Requirements:
# 1)convert it to netcdf4 and write one output variable per file.
# 2)Output files should also pass CF checker and CMIP conventions (and possibly OGC).
# 3) Mars attributes should be clearly identified 
# 4) time should be handled (forecast time, assimilation window etc)
# Use an approach whereby various attributes/additional variables can be added to output NetCDF files by calls to functions
#
# K Marsh 23/9/15
#
# Added option to handle multiple input files, input directory and output directory  
#
# K Marsh 2/6/16
#
# V2 is a rewritten version to streamline the code, as there is now no requirement to split the fields by level
# K Marsh 7/6/16
#
#*****************************************************************
from netCDF4 import Dataset
from netCDF4 import num2date
import datetime
import MySQLdb
import getopt
import sys
import glob


def add_ensemble(fid, ens_num, ens_len):
    fid.createDimension('ensemble', ens_len)
    fid.createVariable('realization', 'i', 'ensemble')
    fid.variables['realization'].setncattr('standard_name', 'realization')
    fid.variables['realization'].setncattr('long_name', "Number of the simulation in the ensemble")
    fid.variables['realization'][:] = ens_num
    return    

def add_mars_atts(fid, att_dict, var):
    st_date = fid.variables['atime'][:]
    date_str, time_str = str(num2date(st_date, fid.variables['atime'].units)).split()
    for item_att in att_dict.keys():
        if item_att == "mars_date" :
            print 'set date', date_str
            fid.variables[var].setncattr(item_att, date_str)
            continue
        if item_att == "mars_time":
            fid.variables[var].setncattr(item_att, time_str)
            continue
        if item_att == "mars_level":
            fid.variables[var].setncattr(item_att, str(fid.variables['deptht'][:]))
            continue        
        fid.variables[var].setncattr(item_att, att_dict[item_att])
    return

def modify_time(fid, var):
    lDebugFlag = True 
    fid.renameVariable('time_counter', 'time')
    fid.renameVariable('time_counter_bnds', 'time_bnds')
    print dir(fid)
    fid.renameDimension('time_counter','time')
    if lDebugFlag: 
        print fid.variables['time_bnds'][:]
    fid.variables['time'].setncattr("bounds", "time_bnds")
    data_var = fid.variables[var]
    for att in data_var.ncattrs():
        val = data_var.getncattr(att)
        print "att,val",att,val
        if "coordinates" in att and "time_counter" in val:
            val_new = val.replace("time_counter", "time")
            data_var.setncattr("coordinates", val_new)
            print "set time_counter to time"
    fid.createVariable('atime', 'd',)
    fid.variables['atime'].setncattr("standard_name", "forecast_reference_time")
    fid.variables['atime'].setncattr("units", fid.variables['time'].getncattr('units'))
    if lDebugFlag:
        print "** **",fid.variables['time_bnds'][0][0],fid.variables['time_bnds'][0][1]
    fid.variables['atime'][:]=fid.variables['time_bnds'][0][0]
    if lDebugFlag:
        print"***** fid atime ", fid.variables['atime'][:],fid.variables['atime'].shape
    return

def add_assimilation_window(fid, var):
    fid.createVariable('assimilation_window_start', 'd',)
    fid.variables['assimilation_window_start'].setncattr("long_name", "assimilation window start")
    fid.variables['assimilation_window_start'].setncattr("units", fid.variables['time'].getncattr('units'))
    fid.variables['assimilation_window_start'][:] = fid.variables['time_bnds'][0, 0]
    fid.createVariable('assimilation_window_end', 'd',)
    fid.variables['assimilation_window_end'].setncattr("long_name", "assimilation window end")
    fid.variables['assimilation_window_end'].setncattr("units", fid.variables['time'].getncattr('units'))
    fid.variables['assimilation_window_end'][:] = fid.variables['time_bnds'][0, 1]
    return

# Python script to perform a simple lookup on the ecmwf param database for a
# given paramID and return the corresponding CF Standard Name. Databases are
# opened as read-only
#


def db_lookup(db_name,host_name,user_name,pass_name,param_num):

    try:
        db = MySQLdb.connect(host = host_name, # your host, usually localhost
                     user = user_name, # your username
                      passwd = pass_name, # your password
                      db = db_name) # name of the data base

    except MySQLdb.Error, e:
  
        print "Error %d: %s" % (e.args[0],e.args[1])
        sys.exit(1)
#
# create a Cursor object to execute all the queries required
#
    cur = db.cursor() 
#
# List parameter from tables in the database
#

    print "Getting CF name for GRIB1 parameter number ", param_num
    paramID = str(param_num)
    query = "SELECT name, grib1_ecmwf FROM cf WHERE grib1_ecmwf = " + paramID
    cur.execute(query)
    desc = cur.description
    print "%s %3s" % (desc[0][0], desc[1][0])
#
# print all the first cell of all the rows
#
    for row in cur.fetchall() :
        print "CF Standard Name: %s  GRIB1 Code: %s \n" % (row[0],row[1])
    db.close()
    return row[0]

def read_config_file(lDebugFlag,cfg_fname):
    print "start!"
    try:
        cfg_file=open(cfg_fname,"r")
    except:
        print "File to process does not exit; Exiting"
        sys.exit()    
    

    lines=cfg_file.readlines()
    m_dict={}
    a_dict={}
    v_dict={}
    fn_list=[]
    o_dir=""
    for line in lines:
        if line.startswith("#") or len(line.strip()) == 0 : # comment/empty lines: 
            continue  
        elif line.startswith("\'"): # "Env: " line
            option, value = line.split("=")
            o=option.strip()
            v=value.strip()
            os.environ[o] ="$"+o+":"+v            
        else:
            if lDebugFlag:
                print line
            option, value = line.split("=")
            o=option.strip()
            v=value.strip()
            if o == "db_name":
                d_name = v
            elif o == "host_name":
                h_name = v
            elif o == "user_name":
                 u_name = v
            elif o == "pass_name":
                p_name =v
            elif o == "mars_type":
                 m_dict["mars_type"] = v                 
            elif o == "mars_class":
                m_dict["mars_class"] = v
            elif o == "mars_stream":
                 m_dict["mars_stream"] = v
            elif o == "mars_expver":
                m_dict["mars_expver"] =v
            elif o == "mars_date":
                 m_dict["mars_date"] = v                  
            elif o == "mars_time":
                m_dict["mars_time"] = v
            elif o == "mars_step":
                 m_dict["mars_step"] = v
            elif o == "mars_number":
                m_dict["mars_number"] =v
            elif o == "mars_levelist":
                 m_dict["mars_levelist"] = v   
            elif o == "mars_levtype":
                m_dict["mars_levtype"] = v
            elif o == "mars_param":
                 m_dict["mars_param"] = v
            elif o == "fnames":
                fn_list.append(v)
            elif o == "input_dir":
                fn_list = glob.glob(v+"\/*.nc")
            elif o == "output_dir": 
                o_dir = v
            elif o == "global_attributes_Conventions":
                 a_dict["Conventions"] = v              
            elif o == "global_attributes_title":
                 at_dict["title"] = v      
            elif o == "global_attributes_institution":
                 a_dict["institution"] = v      
            elif o == "global_attributes_references":
                 a_dict["references"] = v      
            elif o == "global_attributes_source":
                 a_dict["source"] = v  
            elif o == "global_attributes_history":
                 a_dict["history"] = v      
            elif o == "global_attributes_comment":
                 a_dict["comment"] = v                                                      
            elif o == "nemo_var_name":
                 v_dict[v.split(":")[0]]= [v.split(":")[1],v.split(":")[2],v.split(":")[3]]   
            else:
                print "Warning: Unmatched option"
    cfg_file.close()
    if len(fn_list) == 0:
        print "No files to process: exiting"
        sys.exit()
    if o_dir == "":
        print "No output directory specified: exiting"
        sys.exit()      
    if not o_dir.endswith("\/"):
        o_dir =o_dir + "/"
          
    if len(m_dict) != 11:
        print "Missing Mars elements: exiting"
        sys.exit()
    if lDebugFlag:
        print "MARS dictionary is :", m_dict  
    return d_name, h_name,u_name,p_name,m_dict,fn_list,o_dir,a_dict,v_dict




def main():
    """
     main part of the program; take an output NEMO file and split by variable
     
     1 parse config file
     2 get file list
     3 process files in list
     4 get required variables in list
     5 get required axes
     6 get required attributes
     7 get coordinates
     8 get list of timesteps
     9 write out by timesteps
     10 amend variable attributes
     11 amend global attributes
     12 modify hours (NEMO offset)
     13 write out file
     
     
    """
#
# Read configuration file; also build dictionary to hold MARS-specific 
# placeholder metadata 
#
    lDebugFlag = False
    options, remainder = getopt.getopt(sys.argv[1:],'c:')
    if len(options) ==0:
        print "Option required; Exiting"
        sys.exit()
    for opt, arg in options:
        print opt,arg
        if opt in ('-c'):
           cf_cfg_fname = arg
           print "Config file is ",cf_cfg_fname
        else:
            print "Config file required; Exiting"
            sys.exit()
    db_name, host_name,user_name,pass_name,mars_dict,fnames, op_dir,at_d,var_dict = read_config_file(lDebugFlag, cf_cfg_fname)
    print  db_name, host_name,user_name,pass_name,mars_dict,fnames, op_dir,at_d,var_dict
#
# input filename list for testing; input and output directories are taken from the config file
#
    file_count = 0
    for fname in fnames:
        file_count = file_count + 1
        print fname
        fname_out = op_dir+"a"+(fname.split("/")[-1]).split(".")[0]
        dsin = Dataset(fname, "r")
        print fname_out
#
#copy existing file global attributes to output files
#
        nc_global_attrs = dsin.ncattrs()
        print nc_global_attrs
#        
# list of files to process -only those specified in config file with CF standard name       
#        
        var_list = []
        for v_name, varin in dsin.variables.iteritems():
            if v_name in var_dict.keys():
                var_list.append(str(v_name)) 
        print "Variables in file : ",var_list
#
# This is the required list of variables needed in all output 'split' files:
#
        process_list = [item for item in var_list if item in var_dict.keys()]
        print "variables to process : ",process_list
        print len(var_list)," variables in the file "
        print len(var_dict.keys()), " Variables specified in config file"
        print len(process_list)," Variables to process " 
        nvar = 0
        icount = 0
#
# loop over required variables 
#
        for p_name in process_list:
           bounds_list=[]
           icount = icount + 1
#
# get variable from input file
#
           var = dsin.variables[p_name]
           print var
           print dir(var)
           print "processing ", icount, " ", p_name
           print "writing to "+ fname_out + "_reformat_time_cat_" + p_name + ".nc"
           dsout = Dataset(fname_out + "_reformat_time_cat_" + p_name + ".nc", "w", format = "NETCDF3_CLASSIC")
#
# create dimensions; use list of dimension names (strings), and take dimensions from input files (dsin.dimensions)
#
           dim_list = var.dimensions
           str_dim_list = [str(i) for i in dim_list]           
           print "dim list is ",dim_list
           print str_dim_list
           for dname, the_dim in dsin.dimensions.iteritems():
              print dname, len(the_dim),the_dim,dir(the_dim)
              if dname in dim_list or dname == "tbnds": 
                  print dname, "match"
                  dsout.createDimension(dname, len(the_dim) if not the_dim.isunlimited() else None)
#
# Add coordinate variables and auxiliary coordinate variables; 
# loop through variables in file to get required variables
#
           for v_name, varin in dsin.variables.iteritems():
               if v_name in str_dim_list or v_name in var.coordinates or v_name == "time_counter_bnds":
                   print "add coordinate variables", v_name
                   outVar = dsout.createVariable(v_name, varin.datatype, varin.dimensions)
#                  
# Copy coordinate variable attributes
#
                   outVar.setncatts({k: varin.getncattr(k) for k in varin.ncattrs()})
                   for j in varin.ncattrs():
                       if j == "bounds":
                           print v_name, " has bounds"
                           print dir(j)
                   outVar[:] = varin[:]                 
                   print "set coordinate values and attributes for ", v_name          

           print "adding variable ", var._name
           print var._name, var.datatype, var.dimensions
#
# Option to rename netcdf variables to 'data' rather than the NEMO name
#
           set_name_to_data = 1
           if set_name_to_data == 0:
               variable_name=var._name
           if set_name_to_data == 1:
               variable_name = "data"
               NEMO__netcdf_variable_name = var._name
           outVar1 = dsout.createVariable(variable_name, var.datatype, var.dimensions)
           outVar1[:] = var[:]    
#
# Copy variable attributes
#
           for k in var.ncattrs():
#
#ignore title attributes for variables as this contravenes CF
#ignore _FillValue attributes for variables as this is not set correctly
#                    
              if k == "_FillValue" or k == "title": 
                  continue
              if k == "standard_name":
                  outVar1.setncattr("NEMO_standard_name", var.getncattr(k))
                  continue
              if k == "coordinates":
                  outVar1.setncattr(k, "atime " + var.getncattr(k))
                  continue
              outVar1.setncatts({k: var.getncattr(k) })
#
# Todo: set variable fillvalue to original rather than default
#               

#
# if forecast fields, modify time axis accordingly
#
           modify_time(dsout, variable_name)               
           print "output file contents ",dir(dsout.variables)
           print dsout.variables.keys()
#
# Add global attributes from original file
# Delete existing history
#
           print len(nc_global_attrs)
           if len(nc_global_attrs) > 0:
               for ncattr in nc_global_attrs:
                  if ncattr == "history":
                      continue
                  dsout.setncattr(ncattr, nc_global_attrs[ncattr])
#
# Set CF standard names 
#
           cf_name=var_dict[p_name][0]
           print "CF Name is ",cf_name
           grib_param_id = var_dict[p_name][1]
           grib_param_id_str = str(grib_param_id)
           mars_dict['mars_param'] = grib_param_id_str
           add_mars_atts(dsout, mars_dict, "data")
           outVar1.setncattr("standard_name",cf_name)
           dsout.close()
           dsin.close()
           break

"""
#
# This is the required list of variables needed in all output 'split' files:
#
        req_list = ['nav_lon', 'nav_lat','deptht','time_counter', 'time_counter_bnds']
        process_list = [item for item in var_list if item not in req_list]
        nvar = 0
        icount = 0
#
# loop over required variables which are not required in every file
#
        for p_name in process_list:
            icount = icount + 1
#
# loop over variable items in the netcdf file; use string rather than unicode 
#
            for v_name, varin in dsin.variables.iteritems():
                v_name_str = str(v_name)
#
# if a surface filed, skip for now as need to rework the dimensions
#
                if v_name_str[0:2] == "so": 
                    print "skip surface field ", v_name_str
                    continue
#
# if matches a process variable name, open a new file for output
#
                if v_name_str == p_name:
                    print "processing ", icount, " ", v_name_str, type(v_name_str)
                    print "writing to "+ fname_out + "_reformat_time_cat_" + v_name_str + ".nc"
                    dsout = Dataset(fname_out + "_reformat_time_cat_" + v_name_str + ".nc", "w", format = "NETCDF3_CLASSIC")
#
# add global attributes from original file
#
                    if len(nc_global_attrs) > 0:
                        for ncattr in nc_global_attrs:
                            dsout.setncattr(ncattr, nc_global_attrs[ncattr])

#
# write out the dimensions
#
                    for dname, the_dim in dsin.dimensions.iteritems():
                        dsout.createDimension(str(dname), len(the_dim)) 
                    add_ensemble(dsout, file_count, 1)
#
# iterate again to write out the required co-ordinate variables
#
                    for v_name_req, varin_req in dsin.variables.iteritems():        
                        if str(v_name_req) in req_list:
                            if "_FillValue" in varin_req.ncattrs() :                      
                                f_val = getattr(varin_req, "_FillValue")
                                outVar = dsout.createVariable(str(v_name_req), \
			        varin_req.dtype, varin_req.dimensions, fill_value = f_val)
                            else:
                                outVar = dsout.createVariable(str(v_name_req), \
			        varin_req.dtype, varin_req.dimensions)
#
# Writing out required co-ordinate Variable: 
#
                            outVar[:] = varin_req[:]
#
# Have to set variable attributes one at a time; _FillValue was not set correctly; 
# title is not a CF variable attribute
#
                            for ncattr in varin_req.ncattrs():
                                if ncattr == "_FillValue" or ncattr == "title": 
                                    continue
#
# TODO add CF standard name attribute based on NEMO-> CF mapping 
#
                                if ncattr == "standard_name":
                                    outVar.setncattr("NEMO_standard_name", varin_req.getncattr(ncattr))
                                    continue                           
#
# Setting attributes for required co-ordinate variable attribute: 
#
                                outVar.setncattr(ncattr, varin_req.getncattr(ncattr))
#
# write out the 'to process' variable  - v_name_str is the original variable name; 
# _FillValue has to be handled carefully and set in the createVariable call.
#
                    if "_FillValue" in varin.ncattrs():
                        varin1 = ['ensemble',]
                        for x in varin.dimensions:
                            varin1.append(x)
                        f_val = getattr(varin, "_FillValue")
                        outVar = dsout.createVariable("data", varin.dtype, varin1, fill_value = f_val)
                    else:
                        outVar = dsout.createVariable("data", varin.dtype, varin1)
                    print "Writing output variable", v_name_str,varin1, " as \"data\"" 
                    outVar[:] = varin[:] 
#
# Have to set variable attributes one at a time
#
                    outVar.setncattr("NEMO_variable_name", v_name_str)
                    for ncattr in varin.ncattrs():

                    
#
# Using dummy ID -hardcoded in script as not all cf names have grib mapping
# add cf standard_name placeholder
#
                    cf_name=var_dict[v_name][0]
                    print "CF Name is ",cf_name
                    grib_param_id = var_dict[v_name][1]
                    grib_param_id_str = str(grib_param_id)
                    mars_dict['mars_param'] = grib_param_id_str
#                    cf_name=db_lookup(db_name,host_name,user_name,pass_name,grib_param_id_str)
                    add_mars_atts(dsout, mars_dict, "data")
                    outVar.setncattr("standard_name",cf_name)
                    print dsout.dimensions
                    print "closing output file", fname_out
                    dsout.close()
                    f_out=fname_out + "_reformat_time_cat_" + v_name_str + ".nc"
                    print f_out
#                    dsout1 = Dataset(f_out,"a")
#                    print dsout1.data_model
#                    print dsout1.dimensions
#                    dsout1.renameDimension(u'time_counter',u'time')
#                    dsout1.close()
                    nvar = nvar + 1
                    print "Variables processed/total: ",nvar,len(process_list)
  
#
# have processed this 'process' variable, so exit loop and do the next
#
                    if lDebugFlag:
                        break
#
# for testing, just need to output the first two variables from the input file
#
            if lDebugFlag: 
                break
        print "Variables Processed/Total: ", nvar,len(process_list)

"""
if __name__ == "__main__":
    main()







#!/usr/bin/env python3
#
# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

import gdal
import sys

# Build a mask, data from http://landcover.org/data/watermask/index.shtml

"""

Inspect file with gdalinfo

Driver: GTiff/GeoTIFF
Files: MOD44W_Water_2000_FE1718.tif
Size is 5761, 7681
Coordinate System is:
GEOGCS["WGS 84",
    DATUM["WGS_1984",
        SPHEROID["WGS 84",6378137,298.257223563,
            AUTHORITY["EPSG","7030"]],
        AUTHORITY["EPSG","6326"]],
    PRIMEM["Greenwich",0],
    UNIT["degree",0.0174532925199433],
    AUTHORITY["EPSG","4326"]]
Origin = (-84.000000000000000,-47.999997500000006)
Pixel Size = (0.002083333333333,-0.002083333043981)
Metadata:
  AREA_OR_POINT=Area
  TIFFTAG_XRESOLUTION=1
  TIFFTAG_YRESOLUTION=1
Image Structure Metadata:
  INTERLEAVE=BAND
Corner Coordinates:
Upper Left  ( -84.0000000, -47.9999975) ( 84d 0' 0.00"W, 47d59'59.99"S)
Lower Left  ( -84.0000000, -64.0020786) ( 84d 0' 0.00"W, 64d 0' 7.48"S)
Upper Right ( -71.9979167, -47.9999975) ( 71d59'52.50"W, 47d59'59.99"S)
Lower Right ( -71.9979167, -64.0020786) ( 71d59'52.50"W, 64d 0' 7.48"S)
Center      ( -77.9989583, -56.0010381) ( 77d59'56.25"W, 56d 0' 3.74"S)
Band 1 Block=5761x1 Type=Byte, ColorInterp=Gray
"""

"""
360/0.0020833333333333333 = 172800
180/0.0020833333333333333 = 86400


48 20160
-48 66240
-32 58560
-16 50880
0

"""


def s(x):
    return "%5g" % (x,)


header = "P5\n172800\n86400\n255\n"
g = open("p.pgm", "w+b")

lines = []
for i in range(0, 256):
    lines.append(bytearray([i for y in xrange(0, 172800)]))

g.write(header)
for r in xrange(0, 86400):
    g.write(lines[r % 256])

# exit(0)

mapping = {0.0: 0, 1.0: 255, 253: 200}
X = {}
for path in sys.argv[1:]:
    print path
    raster = gdal.Open(path)
    (
        top_left_x,
        w_e_pixel_resolution,
        _,
        top_left_y,
        _,
        n_s_pixel_resolution,
    ) = raster.GetGeoTransform()

    # print "x=", top_left_x, " dx=", w_e_pixel_resolution
    # print "y=", top_left_y, " dy=", n_s_pixel_resolution
    x = raster.GetProjection()
    assert (
        x
        == 'GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433],AUTHORITY["EPSG","4326"]]'
    )

    # print dir(raster)
    assert raster.RasterCount == 1
    band = raster.GetRasterBand(1)
    assert band.GetScale() == 1
    values = band.ReadAsArray()
    (width, height) = values.shape
    print (width, height)

    while top_left_x >= 360:
        top_left_x -= 360

    while top_left_x < 0:
        top_left_x += 360

    col = int(top_left_x * 172800 / 360 + 0.5)
    row = int((90 - top_left_y) * 86400 / 180 + 0.5)

    print (col, row, col + width, row + height)
    print (top_left_x, top_left_y)

    for x in xrange(0, height):
        pos = len(header) + 172800 * (x + row - 1) + col
        assert pos > 0
        g.seek(pos)
        p = [mapping[q] for q in values[x : x + 1].flatten()]
        g.write(bytearray(p))



#!/usr/bin/env python3

#
# This script checks all example-grids and updates the uid to a newly calculated uid
# The only argument to this script is the path to the "atlas-grids" executable
#
# !!! WARNING !!! 
#   Execution overwrites existing ".yml" files in the same directory as this script!
#

def run_program_get_error(file):
    import os
    from subprocess import check_output, CalledProcessError

    command = [program, file, '--check' ]
    
    from subprocess import Popen, PIPE
    
    p = Popen(command, stdout=PIPE, stderr=PIPE)
    output, error = p.communicate()
    if p.returncode != 0:
       return str(error.strip(),'ascii')
    return None


def process_file( file, newfile ):

    import string
    print( "File: " , file )

    f = open(file,"r+")
    text = f.read()
    f.close()

    newtext = text

    lines = run_program_get_error( file )
    if lines:
        for line in lines.splitlines():
            import re
            matched = re.match( r"Check failed: grid uid (.*) expected to be (.*)$", line )
            if matched :
                print( line )
                replace = matched.group(1)
                uid = re.findall( r"^  uid : (.*)$", text, re.MULTILINE)[0]
                print("Search/Replace ",uid, " to ", replace )
                newtext = text.replace( uid, replace )
    else:
        print( "Nothing to replace, copy-only" )

    print( "New file: ", newfile)
    f = open(newfile,"w")
    f.write(newtext)
    f.close()


import sys
program = sys.argv[1]

print( "Updating uids of grids with program [",program,"]")

import os
directory = os.path.dirname(os.path.realpath(__file__))

for root, dirs, files in os.walk(directory):
    for file in files:
        if file.endswith('.yml'):
            file = root+"/"+file
            newfile = file
            process_file(file,newfile)



#!/usr/bin/env python3
#
# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# To run on lxb15
import gribapi
import json
import os

params = {}
SCORES = {}

SCORE = {"od": 10000, "e2": 1000, "e4": 100, "ei": 150, "er": 50, "rd": 10}

names = "shortName,type,stream,paramId,gridType,name,units".split(",")


def dump():
    out = {}
    for k, v in params.items():
        a = out[k] = {}
        for p, q in v.items():
            a[p] = ", ".join(sorted(q))

    with open("/perm/ma/mab/gribs/params.json", "w") as g:
        g.write(json.dumps(out, indent=4))


for root, dirs, files in os.walk("/gpfs/lxab/marsdev/mars_grib2/workdir/class"):

    for name in files:

        if not name.startswith("data."):
            continue

        path = os.path.join(root, name)
        if "/ai/" in path:
            continue

        print(path)
        with open(path) as f:
            while True:
                h = gribapi.grib_new_from_file(f)
                if h is None:
                    break

                id = "%s-%s" % (
                    gribapi.grib_get_string(h, "paramId"),
                    gribapi.grib_get_string(h, "gridType"),
                )
                cl = "%s" % gribapi.grib_get_string(h, "class")
                if id not in params or SCORE.get(cl, 0) > SCORES.get(id, 0):
                    print("New ID", id, gribapi.grib_get_string(h, "name"))
                    with open("/perm/ma/mab/gribs/%s.grib" % (id,), "w") as g:
                        g.write(gribapi.grib_get_message(h))
                    i = gribapi.grib_keys_iterator_new(h, "mars")
                    r = []
                    while True:
                        j = gribapi.grib_keys_iterator_next(i)
                        if not j:
                            break
                        n = gribapi.grib_keys_iterator_get_name(j)
                        r.append("%s=%s" % (n, gribapi.grib_get_string(h, n)))
                    gribapi.grib_keys_iterator_delete(h, i)

                    params.setdefault(id, {})
                    params[id]["mars"] = set([", ".join(r)])
                    SCORES[id] = SCORE.get(cl, 0)

                entry = params[id]

                for n in names:
                    entry.setdefault(n, set())
                    v = "?"
                    try:
                        v = "%s" % gribapi.grib_get_string(h, n)
                        if "unknown" in v:
                            print("unknown", n, "in", path)
                    except Exception:
                        v = "missing"

                    entry[n].add(v.strip())

                gribapi.grib_release(h)

    dump()

dump()



#!/usr/bin/env python3

# Usage: 
#     generate-authors.py > AUTHORS


# Authors
authors = [
    "Willem Deconinck",
    "Pedro Maciel",
    "Tiago Quintino"
]

def real_name(name):
    # Because some contributors use different names or aliases
    alias = {
        "cosunae"      : "Carlos Osuna",
        "carlos osuna" : "Carlos Osuna",
        "mengaldo"     : "Gianmarco Mengaldo"
    }
    if name in alias:
        return alias[name]
    else:
        return name

def contributors():
    # Contributors sorted by number of commits
    from subprocess import check_output
    command=['git', 'shortlog', '-n', '-s']
    output=check_output(command, universal_newlines=True).strip()
    contributors_dict = {}
    for line in output.split('\n'):
        [commits,name] = line.strip().split('\t',1)
        name = real_name(name)
        if name in authors:
            continue
        commits = int(commits)
        if name in contributors_dict:
            commits += contributors_dict[name]
        contributors_dict[name] = commits
    return [item[0] for item in sorted(contributors_dict.items(), key = lambda x: x[1], reverse = True)]

print("Authors")
print("=======\n")
for name in authors:
  print("- "+name)

print("")
print("Thanks for contributions from")
print("=============================\n")

for name in contributors():
  print("- "+name)



#!/usr/bin/env python
#
# -*- coding: utf-8 -*-
# Program to read NEMO ocean model output temperature field file (netCDF3),
# Requirements:
# 1) Convert it to netcdf4 and write one output variable per file.
# 2) Output files should also pass CF checker.
# 3) Mars attributes should be added clearly identified
# 4) Modifications to time should be handled
#
# K Marsh 20/7/16
#
# 
# v3 modified from V2 to take command line input file, output file (prefix),
# type of file to be processed and configuration file to amend attributes etc.
#
# Currently only works for NEMO 3hr and 1D data files - Not 1 month/ non-NEMO data
#
# K Marsh 8/8/16
#
#*****************************************************************
# to run:
# python convert_nemo_to_1_variable_all_levels_one_timestep_v2.py 
# -i copy_of_2366_1d_1900022721_1900022821_grid_T_02.nc 
# -o copy_of_2366_1d_1900022721_1900022821_grid_T_02 
# -k NEMO -c convert_nemo_to_1_variable_all_levels_one_timestep.cfg
# 
# Note that this appends the variable name, timestep and "INTERIM.nc3" 
# to the output filename
#
from netCDF4 import Dataset, netcdftime, num2date
import glob
import sys
import getopt
import datetime
import os

def process_config_file(cfg_file_in):
    islDebugFlag = False
    lines = cfg_file_in.readlines()
    mars_dict = {}
    at_dict = {}
    var_dict = {}
    fnames = []
    op_dir = ""
    for line in lines:
        if line.startswith("#") or len(line.strip()) == 0:#comment/empty lines
            continue
        elif line.startswith("\'"): # "Env: " line
            option, value = line.split("=")
            op = option.strip()
            v = value.strip()
            os.environ[op] = "$" + op + ":" + v
        else:
            if islDebugFlag:
                print line
            option, value = line.split("=")
            op = option.strip()
            v = value.strip()
            if op == "db_name":
                db_name = v
            elif op == "host_name":
                host_name = v
            elif op == "user_name":
                user_name = v
            elif op == "pass_name":
                pass_name = v
            elif op == "mars_type":
                mars_dict["mars_type"] = v
            elif op == "mars_class":
                mars_dict["mars_class"] = v
            elif op == "mars_stream":
                mars_dict["mars_stream"] = v
            elif op == "mars_expver":
                mars_dict["mars_expver"] = v
            elif op == "mars_date":
                mars_dict["mars_date"] = v
            elif op == "mars_time":
                mars_dict["mars_time"] = v
            elif op == "mars_step":
                mars_dict["mars_step"] = v
            elif op == "mars_number":
                mars_dict["mars_number"] = v
            elif op == "mars_levelist":
                mars_dict["mars_levelist"] = v
            elif op == "mars_levtype":
                mars_dict["mars_levtype"] = v
            elif op == "mars_param":
                mars_dict["mars_param"] = v
            elif op == "fnames":
                fnames.append(v)
            elif op == "input_dir":
                fnames = glob.glob(v)
            elif op == "output_dir":
                op_dir = v
            elif op == "global_attributes_Conventions":
                at_dict["Conventions"] = v
            elif op == "global_attributes_title":
                at_dict["title"] = v
            elif op == "global_attributes_institution":
                at_dict["institution"] = v
            elif op == "global_attributes_references":
                at_dict["references"] = v
            elif op == "global_attributes_source":
                at_dict["source"] = v
            elif op == "global_attributes_history":
                at_dict["history"] = v
            elif op == "global_attributes_comment":
                at_dict["comment"] = v
            elif op == "nemo_var_name":
                var_dict[v.split(":")[0]] = \
                [v.split(":")[1], v.split(":")[2], v.split(":")[3], \
                v.split(":")[4], v.split(":")[5]]
            else:
                print "Warning: Unmatched option", op
    cfg_file_in.close()



    if not op_dir.endswith("\/"):
        op_dir = op_dir + "/"
    if len(mars_dict) != 11:
        print "Missing Mars elements: exiting"
        sys.exit()
    if islDebugFlag:
        print "MARS dictionary is :", mars_dict
    return at_dict, mars_dict, var_dict

def get_original_dimensions_and_global_attributes(file_in):
    islDebugFlag0 = True
#
#get dimensions
#
    dim_list = file_in.dimensions
    dim_dict = {}
    nc_attrs = {}
#
#Copy dimensions from whole file
#
    for dname, the_dim in file_in.dimensions.iteritems():
        dim_dict[dname] = len(the_dim)
#
# get global attributes
#
    nc_atts = file_in.ncattrs()
    print nc_atts, type(nc_atts)
    print "NetCDF Global Attributes:"
    print dir(file_in)
    if len(nc_atts) == 0:
        print "No Global attributes in source file"
    if islDebugFlag0:
        for nc_attr in nc_atts:
            print nc_attr
            nc_attrs[nc_attr] = file_in.getncattr(nc_attr)
    return dim_dict, nc_attrs
#
# change existing variable attributes -form a dictionary to 
# hold the ones to be used
#

def modify_variable_attributes(var_in):

    islDebugFlag1 = False
    var_atts_out = {}
    var_atts_to_skip = ["title",]
    for ncattr in var_in.ncattrs():
        if "standard_name" in ncattr:
            var_atts_out["NEMO_standard_name"] = var_in.getncattr(ncattr)
            continue
        if ncattr == "_FillValue":
            f_val = getattr(var_in, "_FillValue")
            var_atts_out[ncattr] = f_val
            continue
        if ncattr == "title":
            continue
        if islDebugFlag1:
            print ncattr
            print var_in.getncattr(ncattr)
        var_atts_out[ncattr] = var_in.getncattr(ncattr)
    if islDebugFlag1:
        print "variable attributes", var_atts_out
    return var_atts_out

def add_mars_atts(fid, att_dict, var):
    islDebugFlag2 = False
    for item_att in att_dict.keys():
        if islDebugFlag2:   
            print item_att
#
#  need to skip mars_param attribute for cases when no mapping?
#
            print item_att, type(item_att)
            print att_dict[item_att], type(att_dict[item_att])
        if item_att in ["mars_levelist"]:
            fid.variables[var].setncattr(item_att, float(att_dict[item_att]))
        elif item_att in ["mars_number", "mars_time", "mars_expver",\
        "mars_date"]:
            fid.variables[var].setncattr(item_att, int(att_dict[item_att]))
        else:
            fid.variables[var].setncattr(item_att, att_dict[item_att])
    return

#
# need to adjust the reference time to that specified in filename
#

def adjust_time(fname_in, time_units_orig):
    islDebugFlag3 = False
    filename_time = (fname_in.split("/")[-1]).split("_")
    time1 = filename_time[2][-2:]
    time2 = filename_time[3][-2:]

    time_units_parts = []
    time_units_parts = time_units_orig.split()
    time_units_parts[-1] = time1 + ":00:00"
    time_units_new = " ".join(time_units_parts)
    if islDebugFlag3:
        print time_units_new

#        sys.exit(1)
    return time_units_new
    
#
# NEMO 1 month averages do not have the correct date/units set.
# These default to the filename and should be set to the first day of each month
# votemper_2366_1m_1900_grid_T_02.nc
# 2366_1d_1900022821_1900030121_grid_T_02.nc
# 2366_3h_1900022821_1900030121_grid_T_03.nc



def add_nemo_1month_date_values(fname_in):
    day_number_list = []
    dates_list = []
    cpts =  fname_in.split("_")
    if cpts[2] == "1m":
        yr_str=cpts[3]
        print yr_str
    else:
        print "wrong 1 month filename; exit"
        sys.exit()
    units_str = "days since "+ yr_str + "-1-1 00:00:00 " 
    print units_str
    dy_str = "01"
    for mn in range(1,13):
        mn_str = str(mn).zfill(2)
        print yr_str, mn_str,dy_str
        date_str = yr_str + "." + mn_str +"."+ dy_str
        fmt = '%Y.%m.%d'
        dt = datetime.datetime.strptime(date_str, fmt)
        print dt
        tt = dt.timetuple()
        day_number_list.append(tt.tm_yday -1)
        dates_list.append(date_str.replace(".",""))
    print day_number_list
    print dates_list


def main():
    add_nemo_1month_date_values("votemper_2366_1m_1900_grid_T_02.nc")

    islDebugFlag = True
    hasConfigFile = False
    hasInputFile = False
    hasOutputFilePath = False
    hasDataType = False 
    print "to run: "
    print "convert_nemo_to_1_variable_all_levels_one_timestep.py " +\
    "-i <input file> -o <output file prefix> -k <data type> -c <config file>"
    print "to Run "
    print "python convert_nemo_to_1_variable_all_levels_one_timestep_v2.py "
    print "-i copy_of_2366_1d_1900022721_1900022821_grid_T_02.nc" 
    print "-o copy_of_2366_1d_1900022721_1900022821_grid_T_02" 
    print "-k NEMO -c convert_nemo_to_1_variable_all_levels_one_timestep.cfg"
 
    print "Note that this appends the variable name, timestep and <INTERIM.nc3>" 
    print "to the output filename"

    print "Parsing input config file"
    options, remainder = getopt.getopt(sys.argv[1:], 'c:i:o:k:')
    print options, remainder
    if len(options) == 0:
        print "Option required; Exiting"
        sys.exit()
    for opt, arg in options:
        print opt, arg
        if opt == '-c':
            cf_cfg_filename = arg
            if not os.path.isfile(cf_cfg_filename):
                print "config file does not exist; exiting"
                sys.exit()
            print "Config file is ", cf_cfg_filename
            hasConfigFile = True
        if opt == '-i':
            input_filename = arg
            if not os.path.isfile(input_filename):
                print "Input file does not exist; exiting"
                sys.exit()
            print "Input file is ", input_filename   
            hasInputFile = True         
        if opt == '-o':   
            output_filename = arg
            if os.path.isfile(output_filename):
                print "Output file does exist and will be overwritten"
            print "Output file is ", output_filename            
            hasOutputFilePath = True            
        if opt == '-k':
            file_type = arg
            if not file_type == 'NEMO':
                print "Need file type to be set (e.g. NEMO)"
                sys.exit()
            print "file type is NEMO"
            hasDataType = True 
        #else:
        #    print "incorrect command line arguement; Exiting"
        #    sys.exit()
    try:
        cfg_file = open(cf_cfg_filename, "r")
    except:
        print "Configuration file to process does not exit; Exiting"


#  Initialise variables to hold file information


    files = []
    at_cfg_dict = {}
    mars_cfg_dict = {}
    var_cfg_dict = {}
    start_date = ""
    start_time = ""
    
    files.append(input_filename)
    print len(files)
    print files
    at_cfg_dict, mars_cfg_dict, var_cfg_dict = \
    process_config_file(cfg_file)

    if islDebugFlag:
        print "at_cfg_dict", at_cfg_dict
        print mars_cfg_dict
        print var_cfg_dict
        print "files to process: "
        print files
        print "\n"

#files = ["/hugetmp/cera20c/split/2366_1d_1900022721_1900022821_grid_T_02.nc",]
    print files, "**"
    if len(files) == 0:
        print "No files to process - exiting !"
        sys.exit()

    for fname in files:
        is3HourFile = False
        is1DayFile = False
        is1MonthFile = False
        fin = Dataset(fname, 'r')
        if "_3hr_" in fname:
            is3HourFile = True
        if "_1m_" in fname:
            is1MonthFile = True
        if "_1d_" in fname:
            is1DayFile = True
        
        file_dims, file_atts = \
        get_original_dimensions_and_global_attributes(fin)
        
        print "Input file Dimensions: ", file_dims
        print "Input file global attributes: ", file_atts

# get list of variables

        fin_var_list = []
        for v in fin.variables:
            if islDebugFlag:
                print v
            fin_var_list.append(v)

        print fin_var_list
        var_count = 0
        for vname in fin_var_list:

# loop through variables

###            if vname == "so14chgt": #temp var to select -use below in operations

            if vname in var_cfg_dict.keys():
                print "match for ", vname
                var = fin.variables[vname]

                for time_step in range(var.shape[0]):
                    var_out_dimensions = {}
                    var_out = var[slice(time_step, time_step+1, None)]
                    dim_count = 0
                    for dim_name in var.dimensions:
                        var_out_dimensions[dim_name] = var_out.shape[dim_count]
                        dim_count = dim_count + 1
                    if islDebugFlag:
                        print time_step
                        print var_out_dimensions
                        print var_out.shape
                        print type(var), type(var_out)

# open output file fname_out used for testing, fname_out1 from config file

                    fname_out = output_filename \
                    + "_" + vname + "_timestep_" + str(time_step) + ".nc"
                    fname_out_temp = fname_out.replace(".nc", "_INTERIM.nc3")
 
                    print "writing INTERIM NC3 file to " + fname_out_temp

                    try:
                        dsout = Dataset(fname_out_temp, "w", \
                        format="NETCDF3_CLASSIC")
                    except:
                        print "Error creating output file ", fname_out_temp
                        print "exiting"
                        sys.exit()

                    for dname, the_dim in var_out_dimensions.iteritems():
                        if dname == "time_counter":
                            dsout.createDimension(dname, size=0)
                        else:
                            dsout.createDimension(dname, the_dim)
                    if "tbnds" in file_dims:
                        dsout.createDimension("tbnds", file_dims["tbnds"])

                     
                        
                    coords = []
                    for co in var.coordinates.split():
                        coords.append(co)
                    for va in var.ncattrs():
                        if va == "coordinates":
                            va_co = var.getncattr(va).split()
                            for v in va_co:
                                if v in coords:
                                    continue
                                else:
                                    coords.append(va_co)

# if a monthly file, manually add time bounds
                    print "is1month file is ", is1MonthFile

                    if is1MonthFile:
                        dsout.createDimension("tbnds", 2)       
                        if "deptht" not in coords:
                            coords.insert(0,"deptht")
                        
                        if "time_counter" not in coords:
                            coords.insert(0,"time_counter")                        

#create time coordinate variables, attributes, set values

                    if islDebugFlag:
                        print "coords", coords
                    hasTimeBounds = False
                    for co_var in coords:
                        if islDebugFlag:
                            print "process", co_var
                        if co_var == "time_counter":
                            if "bounds" in fin.variables[co_var].ncattrs():
                                hasTimeBounds =  True
                                print "set time bounds"
                                time_bounds_var_name = \
                                fin.variables[co_var].getncattr("bounds")

                            outVar = dsout.createVariable("time", \
                            fin.variables[co_var].datatype, \
                            fin.variables[co_var].dimensions)

                            for k in fin.variables[co_var].ncattrs():
                                if k == "bounds":
                                    outVar.setncatts({k: "time_bnds"})
                                elif k == "units":
                                    time_units = \
                                    fin.variables[co_var].getncattr(k)
                                    if islDebugFlag:
                                        print "time units are ", time_units
                                    new_time_units = \
                                    adjust_time(fname, time_units)
                                    outVar.setncattr(k, new_time_units)
                                elif k == "time_origin":
                                    time_origin = \
                                    fin.variables[co_var].getncattr(k)
                                    new_time = \
                                    adjust_time(fname, time_origin)
                                    if islDebugFlag:
                                        print time_origin, new_time, time_origin
                                    new_time_origin_str = \
                                    time_origin.split()[0] + " " + new_time.split()[1]
                                    outVar.setncattr(k, new_time_origin_str)
                                else:
                                    outVar.setncatts({k: \
                                    fin.variables[co_var].getncattr(k)})
                            outVar[:] = fin.variables[co_var][time_step]

# create other coord variables
                        else:
                            outVar = dsout.createVariable(co_var, \
                            fin.variables[co_var].datatype, \
                            fin.variables[co_var].dimensions)
                            outVar.setncatts({k: \
                            fin.variables[co_var].getncattr(k) \
                            for k in fin.variables[co_var].ncattrs()})
                            outVar[:] = fin.variables[co_var][:]
# create time bounds
                    print "hasTimebounds  is ", hasTimeBounds
                    if hasTimeBounds:
                        outVar = dsout.createVariable("time_counter_bnds", \
                        fin.variables["time_counter_bnds"].datatype, \
                        fin.variables["time_counter_bnds"].dimensions)
                        outVar.setncatts({k: \
                        fin.variables["time_counter_bnds"].getncattr(k) \
                        for k in fin.variables["time_counter_bnds"].ncattrs()})

                        outVar[0, :] = \
                        fin.variables["time_counter_bnds"][time_step][0]
                        outVar[:, 1] = \
                        fin.variables["time_counter_bnds"][time_step][1]
                        
                    if is1MonthFile:
                        outVar = dsout.createVariable("time_counter_bnds", \
                        fin.variables["time_counter"].datatype, ("time_counter","tbnds"))
                        

# amend variable attributes

                    var_att_list = modify_variable_attributes(var)
                    if islDebugFlag:
                        print var_att_list
                        print var_cfg_dict
                    layer_needed = False  
                                      
# add any additional coordinate information from the config file
# add scalar dimension and scalar coordinate variable (depth) if required for layer integral

                    if var_cfg_dict[vname][3] != "None":
                        scalar_cpts = var_cfg_dict[vname][3].split("_")
                        scalar_name = scalar_cpts[0]
                        scalar_units = scalar_cpts[2]
                        scalar_value = scalar_cpts[1]
                        scalar_long_name = var_cfg_dict[vname][4]
                        if islDebugFlag:
                            print scalar_cpts

                        if scalar_name == "depth":
                            if islDebugFlag:
                                print "found depth layer"
                            layer_needed = True
                            dsout.createDimension(scalar_name, size=1)
                            outvar=dsout.createVariable(scalar_name,'f4', )
                            outvar[:] = float(scalar_value)
                            outvar.units = scalar_units
                            outvar.positive = "down"
                            outvar.bounds = "depth_bnds"
                            outvar.long_name = scalar_long_name
                            var_att_list["coordinates"] = \
                            var_att_list["coordinates"].replace("time_counter", "time_counter "\
                             + scalar_name)
                            dsout.createDimension("dbnds", size=2)
                            outvar=dsout.createVariable("depth_bnds", 'f4', ("depth", "dbnds"))
                            outvar[0, 0] = 0 
                            outvar[0, 1] = float(scalar_value)

# add scalar variable if required for coordinate                            

                        if layer_needed is False:  
                            if "." in scalar_value:
                                dsout.createVariable(scalar_name, 'f4', )
                                scalar_var = dsout.variables[scalar_name]
                                scalar_var[:] = float(scalar_value)
                            else:
                                dsout.createVariable(scalar_name, 'i4', )
                                scalar_var = dsout.variables[scalar_name]
                                scalar_var[:] = int(scalar_value)
                            scalar_var.units = scalar_cpts[2]
                            scalar_var.long_name = scalar_long_name
                            var_att_list["coordinates"] = \
                            scalar_name + " " + var_att_list["coordinates"]

# Create variable, attributes, set values
                    if layer_needed:
                        if len(var.dimensions) == 3:
                            depth_layer_dims = (var.dimensions[0], "depth", \
                            var.dimensions[1], var.dimensions[2])
                        else:
                            print "dimension length problem for layer"
                            sys.exit()
                        outVar = \
                    dsout.createVariable(vname, var.datatype, depth_layer_dims)
                    else:
                        outVar = \
                        dsout.createVariable(vname, var.datatype, var.dimensions)

                    if "deptht" in var_att_list["coordinates"]:
                        print "found depth info for " + vname 
                        var_att_list["coordinates"] = \
                        var_att_list["coordinates"].replace("deptht", "depth")
                        
                    if "time_counter" in var_att_list["coordinates"]:
                        var_att_list["coordinates"] = \
                        var_att_list["coordinates"].replace("time_counter", \
                        "time")

                    for v in var_att_list:
                        if islDebugFlag:
                            print "* ", v, var_att_list[v]

                        outVar.setncattr(v, var_att_list[v])

# cf standard name
                    if len(var_cfg_dict[vname][0]) > 0:
                        cf_name = var_cfg_dict[vname][0]
                        outVar.setncattr("standard_name", cf_name)
                        outVar.setncattr("NEMO_variable_name", vname)
# cf standard units
                    if len(var_cfg_dict[vname][1]) > 0:
                        var_units = var_att_list["units"]
                        cf_units = var_cfg_dict[vname][1]
                        outVar.setncattr("units", cf_units)
                        outVar.setncattr("NEMO_units", var_units)

#  Add cell methods for variable
                    isCellMethodsBounds = False
                    if "1d" in fname:
                        outVar.setncattr("cell_methods", "time: mean (interval: 1.0 day)")
                    if "3h" in fname:
                        outVar.setncattr("cell_methods", "time: mean (interval: 3.0 hours)")                        
                    if isCellMethodsBounds:
                        outVar.setncattr("cell_methods", "time: mean (over time bounds)")

                    outVar[:] = var_out[:]
# write global attributes
                    if islDebugFlag:
                        print len(file_atts)
                    if len(file_atts) > 0:
                        print file_atts
                        for ncattr in file_atts:
                            if ncattr == "history":
                                continue
                            print "***",ncattr, file_atts[ncattr]
                            dsout.setncattr(ncattr, file_atts[ncattr])

# write global attributes from config file

                    for key in at_cfg_dict:
                        dsout.setncattr(key, at_cfg_dict[key])

# write MARS specific attributes amending elements to those calculated from file
                    print dsout.variables
                    cdftime = netcdftime.utime(dsout.variables["time"].units)
                    if hasTimeBounds:
                        start_date_time = \
                        str(cdftime.num2date\
                        (float(dsout.variables["time_counter_bnds"][0][0]))).split()
                    else:
                        start_date_time = str(cdftime.num2date\
                        (fin.variables["time_counter"][time_step])).split()
                        print start_date_time
                    m_date = start_date_time[0].replace("-", "")
                    m_time = "".join(start_date_time[1].split(":")[:-1])
                    mars_cfg_dict["mars_date"] = m_date
                    mars_cfg_dict["mars_time"] = m_time

# need to adjust mars_level and levelist as required
                    #mars_cfg_dict["mars_level"] = ""
                    #mars_cfg_dict["mars_levelist"] =""
                    add_mars_atts(dsout, mars_cfg_dict, vname)
                    if "time_counter" in dsout.dimensions:
                        dsout.renameDimension('time_counter', 'time')
                    if "time_counter_bnds" in dsout.variables:
                        dsout.renameVariable('time_counter_bnds', 'time_bnds')
                    if "deptht" in dsout.dimensions:
                        dsout.renameDimension('deptht', 'depth')
                    if "deptht" in dsout.variables:
                        dsout.renameVariable('deptht', 'depth')
                    var_count = var_count + 1
                    dsout.close()
        fin.close()
        print "Processed: ", fname
        print "No. of Variables in file: ", len(fin_var_list)
        print "No. of Variables processed: ", var_count

if __name__ == "__main__":
    main()



def compute_deaccumulation(params, data, target_shortname):

    if len(params) == 0 or len(data) == 0 :
        return None

    param = params[0]
    fs = mv.read(data = data, param = param)

    if len(fs) == 0 :
        return None

    # set values in first timestep to 0 and add to result
    res = fs[0] * 0

    # deaccumulate fieldset
    num = len(fs)
    res.append(fs[1:num-1] - fs[0:num-2])

    # set correct shortname for result
    res = mv.grib_set_string(res, ["shortName", target_shortname])

	# mark as non-derived field - so that the correct scaling could be applied in contouring
    res = mv.grib_set_long(res, ["generatingProcessIdentifier", 149])
    
    return res



#!/usr/bin/env python3
#
# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
#
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# To run on lxb15
import json

names = [
    ("Name", "name",),
    ("Abbr.", "shortName",),
    ("Type", "type",),
    ("Stream", "stream",),
    ("ID", "paramId",),
    ("Grid", "gridType",),
    ("Units", "units",),
    ("Method", None),
    ("Comment", None),
    ("Who", None),
    ("Request", "mars",),
]

with open("/perm/ma/mab/gribs/params.json") as f:
    params = json.loads(f.read())

print(
    """
<table>
<tbody>
"""
)

print("<tr>")
for n in names:
    print("<th>%s</th>" % (n[0],))

print("</tr>")

for k in sorted(params.keys(), key=lambda a: params[a]["name"]):
    p = params[k]
    print("<tr>")
    for n in names:
        v = p.get(n[1], "")
        print("<td>%s</td>" % (v),)
    print()
    print("</tr>")

print(
    """
</tbody>
</table>

"""
)




import xml.sax
import requests
import json
import sys, getopt
import os

epsgrams = {
    "15_days"           : "classical_15d",
    "15_days_with_clim" : "classical_15d_with_climate",
    "10_days"           : "classical_10d",
    "10_days_wave"      : "classical_wave",
    "10_days_plumes"    : "classical_plume",

}

class MetgramHandler( xml.sax.ContentHandler ):
   def __init__(self):
      self.CurrentData = ""
      self.type = ""
      self.date = ""
      self.expver = ""
      self.stations = []


   # Call when an element starts
   def startElement(self, tag, attributes):
      self.CurrentData = tag
      if tag == "eps":
         self.type = attributes["template"]
         self.date = attributes.get("date", "").strip()
         self.time = attributes.get("time", "").strip()
         if ( self.time != "") :
            self.time = self.time[:2]
         self.expver = attributes.get("expver", "0001")
         self.stations = []
         if self.date == "latest":
            self.date = ""
         else:
            self.date = "%s%s" % (self.date, self.time)


      if tag == "station":

         request =  {
            "station_name" :  attributes["name"],
            "lat" :  attributes["latitude"].strip(),
            "lon" :  attributes["longitude"].strip(),
            "token" : "metview",
            "expver" : self.expver,
            "epsgram" : epsgrams[self.type]
         }
         if attributes.has_key("height") :
            request["altitude"] =  attributes["height"],

         if self.date != "" :
            request["time"] = self.date

         if attributes.has_key("pngfile") :
            request["format"] = "png"
            output =  attributes["pngfile"]
         if attributes.has_key("psfile") :
            request["format"] = "ps"
            output =  attributes["psfile"]
         if attributes.has_key("pdffile") :
            request["format"] = "pdf"
            output =  attributes["pdffile"]
         self.stations.append({"request" : request,
                "output": output}
            )



   # Call when an elements ends
   def endElement(self, tag):
      if tag == "eps":


         self.execute()



   # Call when a character is read
   def characters(self, content):
      pass

   def execute(self):
        url = "https://apps.ecmwf.int/plots/product-download/web/classical_meteogram"
        for station in self.stations :
            request = station["request"]
            request["token"]           = "metview" # this token will only work from inside ECMWF
            request["email"]           = "Web.Administrator@ecmwf.int"
            request["generator"]       = "Metview"
            request["metview_version"] = os.environ["METVIEW_RELEASE"]
            request["metview_user"]    = os.environ["USER"]
            request = requests.get(url, params = request,
            proxies = { "http":  "http://proxy.ecmwf.int:3333/", 
                        "https": "http://proxy.ecmwf.int:3333/" })
            print request.url
            if request.status_code == 200 :
                out = open(station["output"], "w")
                out.write(request.content)
                out.close()
                print "Result saved in %s" % (station["output"], )
            else :
                print "No Result -->Error %d" % (request.status_code)
        self.stations = []


def main(argv):
    try:
      opts, args = getopt.getopt(argv,"hi:",["input="])
    except getopt.GetoptError:
      print 'metgram.py -i <inputfile> '
      sys.exit(2)
    for opt, arg in opts:
      if opt == '-h':
         print 'metgram.py -i <inputfile> '
         sys.exit()
      elif opt in ("-i", "--input"):
         input = arg

   # create an XMLReader
    parser = xml.sax.make_parser()
   # turn off namepsaces
    parser.setFeature(xml.sax.handler.feature_namespaces, 0)

   # override the default ContextHandler
    Handler = MetgramHandler()
    parser.setContentHandler( Handler )

    parser.parse(input)

if __name__ == "__main__":
   main(sys.argv[1:])
